[tor-commits] [tor/master] Actually release buffer freelists when handling OOM conditions.

nickm at torproject.org nickm at torproject.org
Tue Mar 4 16:04:42 UTC 2014


commit fd28754dd3dce0e00304825d531348414c0a354b
Author: Nick Mathewson <nickm at torproject.org>
Date:   Sun Jan 12 12:26:45 2014 -0500

    Actually release buffer freelists when handling OOM conditions.
    
    Otherwise freeing buffers won't help for a little while.
---
 src/or/buffers.c     |    9 +++++++--
 src/or/buffers.h     |    2 +-
 src/or/circuitlist.c |   13 +++++++++++++
 3 files changed, 21 insertions(+), 3 deletions(-)

diff --git a/src/or/buffers.h b/src/or/buffers.h
index 677d68d..b3b9003 100644
--- a/src/or/buffers.h
+++ b/src/or/buffers.h
@@ -18,7 +18,7 @@ void buf_free(buf_t *buf);
 void buf_clear(buf_t *buf);
 buf_t *buf_copy(const buf_t *buf);
 void buf_shrink(buf_t *buf);
-void buf_shrink_freelists(int free_all);
+size_t buf_shrink_freelists(int free_all);
 void buf_dump_freelist_sizes(int severity);
 
 size_t buf_datalen(const buf_t *buf);
diff --git a/src/or/circuitlist.c b/src/or/circuitlist.c
index 2e13541..ffb5e0c 100644
--- a/src/or/circuitlist.c
+++ b/src/or/circuitlist.c
@@ -1533,6 +1533,17 @@ circuits_handle_oom(size_t current_allocation)
              "MaxMemInQueues.)");
 
   {
+    const size_t recovered = buf_shrink_freelists(1);
+    if (recovered >= current_allocation) {
+      log_warn(LD_BUG, "We somehow recovered more memory from freelists "
+               "than we thought we had allocated");
+      current_allocation = 0;
+    } else {
+      current_allocation -= recovered;
+    }
+  }
+
+  {
     size_t mem_target = (size_t)(get_options()->MaxMemInQueues *
                                  FRACTION_OF_DATA_TO_RETAIN_ON_OOM);
     if (current_allocation <= mem_target)
@@ -1575,6 +1586,8 @@ circuits_handle_oom(size_t current_allocation)
   } SMARTLIST_FOREACH_END(circ);
 
   clean_cell_pool(); /* In case this helps. */
+  buf_shrink_freelists(1); /* This is necessary to actually release buffer
+                              chunks. */
 
   log_notice(LD_GENERAL, "Removed "U64_FORMAT" bytes by killing %d circuits.",
              U64_PRINTF_ARG(mem_recovered),





More information about the tor-commits mailing list
diff --git a/src/or/buffers.c b/src/or/buffers.c
index 856c9b6..352e60a 100644
--- a/src/or/buffers.c
+++ b/src/or/buffers.c
@@ -276,12 +276,14 @@ preferred_chunk_size(size_t target)
 }
 
 /** Remove from the freelists most chunks that have not been used since the
- * last call to buf_shrink_freelists(). */
-void
+ * last call to buf_shrink_freelists().   Return the amount of memory
+ * freed. */
+size_t
 buf_shrink_freelists(int free_all)
 {
 #ifdef ENABLE_BUF_FREELISTS
   int i;
+  size_t total_freed = 0;
   disable_control_logging();
   for (i = 0; freelists[i].alloc_size; ++i) {
     int slack = freelists[i].slack;
@@ -315,6 +317,7 @@ buf_shrink_freelists(int free_all)
         chunk_t *next = chunk->next;
         tor_assert(total_bytes_allocated_in_chunks >= CHUNK_ALLOC_SIZE(chunk->memlen));
         total_bytes_allocated_in_chunks -= CHUNK_ALLOC_SIZE(chunk->memlen);
+        total_freed += CHUNK_ALLOC_SIZE(chunk->memlen);
         tor_free(chunk);
         chunk = next;
         --n_to_free;
@@ -343,8 +346,10 @@ buf_shrink_freelists(int free_all)
   }
  done:
   enable_control_logging();
+  return total_freed;
 #else
   (void) free_all;
+  return 0;
 #endif
 }
 